'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var $ = {},
    del = void 0,
    inky = void 0,
    siphon = void 0,
    lazypipe = void 0,
    gutil = void 0,
    async = void 0,
    deasync = void 0;

var FoundationEmailsTask = function (_Elixir$Task) {
    _inherits(FoundationEmailsTask, _Elixir$Task);

    /**
     * Create a new JavaScriptTask instance.
     *
     * @param  {string}      name
     * @param  {object|null} options
     */
    function FoundationEmailsTask(name, options) {
        _classCallCheck(this, FoundationEmailsTask);

        var _this = _possibleConstructorReturn(this, (FoundationEmailsTask.__proto__ || Object.getPrototypeOf(FoundationEmailsTask)).call(this, name, null, null));

        _this.options = options;

        if (_fs2.default.existsSync('foundation-emails.config.js')) {
            _this.userFoundationEmailsConfig = require(process.cwd() + '/foundation-emails.config.js');
        }

        _this.config = _this.mergeConfig();
        _this.config.sassFilename = _path2.default.posix.basename(_this.config.sass, '.scss');

        _this.src = [];
        _this.output = [];
        return _this;
    }

    /**
     * Lazy load the task dependencies.
     */


    _createClass(FoundationEmailsTask, [{
        key: 'loadDependencies',
        value: function loadDependencies() {
            $.if = require('gulp-if');
            $.sourcemaps = require('gulp-sourcemaps');
            $.sass = require('gulp-sass');
            $.imagemin = require('gulp-imagemin');
            $.postcss = require('gulp-postcss');
            $.uncss = require('postcss-uncss');
            $.htmlmin = require('gulp-htmlmin');
            $.inlineCss = require('gulp-inline-css');
            $.replace = require('gulp-replace');

            del = require('del');
            inky = require('inky');
            siphon = require('siphon-media-query');
            lazypipe = require('lazypipe');
            gutil = require('gulp-util');
            async = require('async');
            deasync = require('deasync');
        }

        /**
         * Register file watchers.
         */

    }, {
        key: 'registerWatchers',
        value: function registerWatchers() {
            if (Elixir.isWatching()) {
                var self = this;
                Elixir.hooks.watch.push(function () {
                    self.watch();
                });
            }
        }

        /**
         * Build up the Gulp task.
         *
         * callback don't have a callback(done), so need to use deasync to run this stream inside the gulp task
         */

    }, {
        key: 'gulpTask',
        value: function gulpTask() {
            var self = this;
            var sync = true;

            self.clean();

            async.series([function (next) {
                self.pages().on('error', function (err) {
                    gutil.log(err);
                    self.onError()(err);
                }).on('end', next);
            }, function (next) {
                self.sass().on('error', function (err) {
                    gutil.log(err);
                    self.onError()(err);
                }).on('end', next);
            }, function (next) {
                self.images().on('error', function (err) {
                    gutil.log(err);
                    self.onError()(err);
                }).on('end', next);
            }, function (next) {
                self.inline().on('error', function (err) {
                    gutil.log(err);
                    self.onError()(err);
                }).on('end', next);
            }, function (next) {
                // compile output paths
                self.output = self.output.map(function (file) {
                    if (file.match(/\*/) || _fs2.default.existsSync(file)) {
                        return gutil.colors.green(file);
                    }

                    return gutil.colors.bgRed(file);
                }).join('\n');
                next();
            }], function () {
                gulp.src('').pipe(new Elixir.Notification('Foundation emails Compiled!')).on('end', function () {
                    sync = false;
                }).emit('end');
            });

            while (sync) {
                deasync.sleep(100);
            }

            return true;
        }

        /**
         * Merge the FoundationEmails config.
         *
         * @return {object}
         */

    }, {
        key: 'mergeConfig',
        value: function mergeConfig() {
            return (0, _lodash.mergeWith)(Elixir.foundationEmails.config, this.userFoundationEmailsConfig, this.options, function (objValue, srcValue) {
                if ((0, _lodash.isArray)(objValue)) {
                    return objValue.concat(srcValue);
                }
            });
        }

        // Delete the "dist" folder
        // This happens every time a build starts

    }, {
        key: 'clean',
        value: function clean() {
            var config = this.config;

            this.recordStep('Clean compiled');

            del.sync([config.compiled, config.views, config.publicCss + '/' + config.sassFilename + '.css', config.imagesDist]);
        }

        // Compile layouts, pages, and partials into flat HTML files
        // Then parse using Inky templates

    }, {
        key: 'pages',
        value: function pages() {
            this.recordStep('Compiling Templates');
            var config = this.config;

            this.addSrcPath(config.source + '/**/*.blade.php');
            this.addOutputPath(config.views);

            return gulp.src(config.source + '/**/*.blade.php').pipe(inky()).pipe($.replace('-&gt;', '->')).pipe($.replace('=&gt;', '=>')).pipe($.replace('&quot;', '"')).pipe($.replace('&apos;', '\'')).pipe(gulp.dest(config.views));
        }

        // Compile Sass into CSS

    }, {
        key: 'sass',
        value: function sass() {
            this.recordStep('Compiling Sass');
            var config = this.config;

            this.addSrcPath(config.sass);
            this.addOutputPath(config.compiled + '/css/' + config.sassFilename + '.css');
            if (!Elixir.config.production) {
                this.addOutputPath(config.publicCss + '/' + config.sassFilename + '.css');
            }

            return gulp.src(config.sass).pipe($.if(Elixir.config.sourcemaps, $.sourcemaps.init())).pipe($.sass({
                errLogToConsole: true
            }).on('error', $.sass.logError)).pipe($.if(Elixir.config.production, $.postcss([$.uncss({
                html: [config.views + '/**/*.blade.php']
            })]))).pipe($.if(Elixir.config.sourcemaps, $.sourcemaps.write())).pipe($.if(!Elixir.config.production, gulp.dest(config.publicCss))).pipe(gulp.dest(config.compiled + '/css'));
        }

        // Copy and compress images

    }, {
        key: 'images',
        value: function images() {
            this.recordStep('Minifing Images');
            var config = this.config;

            this.addSrcPath(config.images + '/**/*');
            this.addOutputPath(config.imagesDist);

            return gulp.src(config.images + '/**/*').pipe($.imagemin()).pipe(gulp.dest(config.imagesDist));
        }

        // Inline CSS and minify HTML

    }, {
        key: 'inline',
        value: function inline() {
            var config = this.config;

            if (Elixir.config.production) {
                this.recordStep('Inlining Css');
            }

            return gulp.src(config.views + '/**/*.blade.php').pipe($.if(Elixir.config.production, this.inliner(config.compiled + '/css/' + config.sassFilename + '.css'))).pipe(gulp.dest(config.views));
        }

        // Inline CSS into HTML, adds media query CSS into the <style> tag of the email, and compresses the HTML

    }, {
        key: 'inliner',
        value: function inliner(css) {
            css = _fs2.default.readFileSync(css).toString();
            var mqCss = siphon(css);

            var additionalCss = '';
            var blade = [];

            var pipe = lazypipe().pipe($.replace, /<style type="text\/css" media="screen">([\s\S]+?)<\/style>/g, function (match, p1) {
                additionalCss += "\n" + p1;
                return '';
            }).pipe($.replace, /<link rel="stylesheet" type="text\/css" href="[^"]+?">/, '').pipe($.replace, /(\{\{.+?\}\}|\{!!.+?!!\})/g, function (match, p1) {
                blade.push(p1);
                return '@__@gulp@blade__@';
            }).pipe($.replace, '<!-- <style> -->', '@__@gulp@style__@').pipe($.inlineCss, {
                applyStyleTags: false,
                removeStyleTags: true,
                preserveMediaQueries: true,
                removeLinkTags: false,
                extraCss: css
            }).pipe($.htmlmin, {
                collapseWhitespace: true,
                minifyCSS: true
            }).pipe($.replace, '@__@gulp@style__@', function () {
                return '<style>' + additionalCss + "\n" + mqCss + '</style>';
            }).pipe($.replace, /(@__@gulp@blade__@)/g, function () {
                return blade.shift();
            });

            return pipe();
        }

        // Watch for file changes

    }, {
        key: 'watch',
        value: function watch() {
            this.isWatching = true;

            var self = this;
            var config = this.config;
            var sassDir = _path2.default.posix.dirname(config.sass);

            // watch blade
            gulp.watch(config.source + '/**/*.blade.php').on('change', function () {
                async.series([function (next) {
                    self.pages().on('error', self.onError).on('end', next);
                }], function () {
                    gulp.src('').pipe(new Elixir.Notification('Foundation emails templates Compiled!')).emit('end');
                });
            });

            // watch sass
            gulp.watch(sassDir + '/**/*.scss').on('change', function () {
                async.series([function (next) {
                    self.pages().on('error', self.onError).on('end', next);
                }, function (next) {
                    self.sass().on('error', self.onError).on('end', next);
                }, function (next) {
                    self.inline().on('error', self.onError).on('end', next);
                }], function () {
                    gulp.src('').pipe(new Elixir.Notification('Foundation emails Compiled!')).emit('end');
                });
            });

            // watch images
            gulp.watch(config.images + '/**/*').on('change', function () {
                async.series([function (next) {
                    self.images().on('error', self.onError).on('end', next);
                }], function () {
                    gulp.src('').pipe(new Elixir.Notification('Foundation emails images minified!')).emit('end');
                });
            });
        }
    }, {
        key: 'addSrcPath',
        value: function addSrcPath(path) {
            if (!this.isWatching) this.src.push(path);
        }
    }, {
        key: 'addOutputPath',
        value: function addOutputPath(path) {
            if (!this.isWatching) this.output.push(path);
        }
    }]);

    return FoundationEmailsTask;
}(Elixir.Task);

exports.default = FoundationEmailsTask;